/* 
 * File:   Vector.hh
 * Author: ngo
 *
 * Created on October 17th, 2010
 */

#ifndef _VECTOR_HH
#define	_VECTOR_HH
#include <vector>
#include <cmath>
#include <assert.h>
#include <iostream>
#include <sstream>
#include <iomanip>

class Vector : public std::vector<double> {
public:
  
  // constructors:
  Vector() : std::vector<double>() {
  }
  
    Vector(const int size, const double defaultvalue_ = 0)
    : std::vector<double>( size, defaultvalue_ ) {
  }
  /*
  Vector(const int dim, const double* array_){
    for(size_t i=0; i<dim; i++){
      double a = array_[i];
      this->push_back(a);
    }
  }
  */

  // inner product with another vector:
  double operator*(Vector & x) const {
    assert( x.size() == this->size() );
    double sum( 0 );
    const Vector & self = *this;
    for( size_t i = 0; i < this->size(); ++i )
      sum += self[i] * x[i];
    return sum;
  }

  // square of the euclidean norm:
  double two_norm_sqr() const {
    double sum( 0 );
    const Vector & self = *this;
    for (int i = 0; i < (int) this->size(); ++i)
      sum += self[i] * self[i];
    return sum;
  }

  // euclidean norm:
  double two_norm() const {
    return sqrt(two_norm_sqr());
  }

 
  // x = x + alpha * y :
  Vector & scaled_add(const double alpha, const Vector & y) {
    assert( this->size() == y.size());
    Vector &self = *this;
    for (size_t i = 0; i < this->size(); ++i)
	self[i] += alpha * y[i];
    return *this;
  }


  // x = alpha * x + y
  Vector & a_times_x_plus_y(const double alpha, const Vector & y) {
    assert( this->size() == y.size());
    Vector & self = *this;
    for (size_t i = 0; i < this->size(); ++i)
      self[i] = alpha * self[i] + y[i];
    return *this;
  }


  void print(void) const {
    const Vector &self = *this;
    for (size_t i = 0; i < self.size(); i++) {
      std::cout << self[i] << "  ";
    }
    std::cout << std::endl;
    std::cout << std::endl;
  }
};


std::ostream & operator <<(std::ostream & os, const Vector & x) {

  os << std::endl;
  //os << std::scientific;

  for (int r = 0; r < (int) x.size(); ++r) {
    os << std::setw(10) << std::setprecision(8) << x[r] << std::endl;
  }
  return os;
}




#endif	/* _VECTOR_HH */

